#!/bin/sh
# This is part of the acl-installer program: 
#    http://kevin.rosenberg.net/acl-installer
#
# Copyright (c) 2002 Kevin M. Rosenberg
# Copyright (c) 2000-2002 Peter Van Endye
#
# acl-installer is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License (version 2) as
# published by the Free Software Foundation.
#
# acl-installer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have a copy of the GNU General Public License on your
# Debian system in the file /usr/share/common-licenses/GPL-2

 
if [ ! -f /usr/share/common-lisp/source/common-lisp-controller/common-lisp-controller.lisp ] ; then
  echo "*** Can't find common-lisp-controller.lisp ***" >&2
  echo "Please report this as a bug" &>2
  exit 1
fi

acl_dir=/usr/lib/acl-pro
lisp_builder=mlisp
lisp_target=mlisp
clc_lib_dir=/usr/lib/common-lisp/$lisp_target

builder=$acl_dir/$lisp_builder
old_dxl=$acl_dir/$lisp_target.dxl
new_dxl=$acl_dir/${lisp_target}-new.dxl
orig_dxl=$acl_dir/${lisp_target}orig.dxl
lpt=$acl_dir/logical-translations-${lisp_target}.dat

if [ -z "$acl_dir" -o -z "$lisp_builder" -o -z "$lisp_target" ]; then
  echo "Error: variables not set correctly in $0" >&2
  exit 1;
fi

lisp_error ()
{
    echo "Error running $builder" >&2
    exit 1
}

dxl_error ()
{
    echo "Error moving new lisp image $new_dxl" >&2
    exit 1
}


case $1 in
    rebuild)
	echo $0 Rebuilding packages...
	shift
	while [ -x $acl_dir/$lisp_builder -a ! -z "$1" ] ; do
	    echo ...rebuilding $1
	    /usr/bin/$lisp_target -q -batch -L $acl_dir/siteinit.cl -e "
(let ((*compile-print* nil)
      (*compile-progress* nil)
      (*compile-verbose*  nil)
      (*require-verbose* nil)
      (*load-verbose* nil) 
      (mk::*load-source-if-no-binary* nil)
      (mk::*bother-user-if-no-binary* nil)
      (mk::*compile-during-load* t))
   (handler-case
      (progn
         (load \"$acl_dir/siteinit.cl\")
         (c-l-c:read-all-logical-pathnames)
         (c-l-c:compile-library (quote $1))
         (c-l-c:write-all-logical-pathnames)
         (excl:exit 0)
       )
    (error (e)
      (ignore-errors (format t \"~&Build error: ~A~%\" e))
      (finish-output)
      (excl:exit 1))))" -kill || lisp_error
	    shift
	done 
	chown cl-builder.cl-builder $lpt
	chmod 0644 $lpt
	;;
    remove)
	echo $0 Removing packages...
	shift
	while [ ! -z "$1" ] ; do
	    echo ...removing package $1
	    rm -rf "${clc_lib_dir}/$1"
	    shift
	done
	rmdir $clc_lib_dir 2> /dev/null 
	;;
    install-clc)
	echo Installing clc...
	if [ -x $acl_dir/$lisp_builder ]; then
	    $builder -q -batch -I $old_dxl \
		-L $acl_dir/siteinit.cl -e "
(handler-case
  (progn 
    (when (find-package :c-l-c) ; have to remove 
      (delete-package :c-l-c))  ; for acl workaround
    (load \"$acl_dir/siteinit.cl\")
    (load \"$acl_dir/install-clc-${lisp_target}.cl\") 
    (dumplisp :name \"${new_dxl}\")
    (excl:exit 0))
  (error (e)
    (ignore-errors (format t \"~&install-clc error: ~A~%\" e))
    (finish-output)
    (excl:exit 1)))" -kill > /dev/null  || lisp_error
  	    mv $new_dxl $old_dxl || dxl_error
	    chown cl-builder.cl-builder $lpt
	fi
	;;
    remove-clc)
	if [ -f $orig_dxl ]; then
	    cp $orig_dxl $old_dxl
	    # Skip calling dl-and-install-patches for speed's sake
	    # So, after remove-clc, the original distributed
	    # image will be active without any of the downloaded patches.
	    # It will be up to the user to manual build images with patches
	    # if they want to run without install-clc
	    # ${acl_dir}/dl-and-install-patches.sh remove-clc
	else
	    echo "Warning: Can't find original image file $orig_dxl. Aborting." >& 2
	fi
	;;
    make-user-image)
	if [ ! -f $2 ] ; then 
	    echo "Trying to make-user image, but can not find file $2" >&2
	    exit 1
	fi 
	$builder -q -batch -I $old_dxl -L $acl_dir/siteinit.cl \
	    -e "
(handler-case
  (progn    
    (c-l-c:read-all-logical-pathnames)
    (load \"$2\") 
    (dumplisp :name \"${new_dxl}\")
    (c-l-c:write-all-logical-pathnames)
    (excl:exit 0))
  (error (e)
    (ignore-errors (format t \"make-user-image error: ~A~%\" e))
    (finish-output)
    (excl:exit 1)))" -kill > /dev/null || lisp_error
	mv $new_dxl $old_dxl || dxl_error
	chown cl-builder.cl-builder $lpt
	;;
    *)
	echo "$0: Unknown command $1" >&2
	echo "Known commands are:" >&2 
	echo "install-clc, remove-clc, rebuild, remove, and make-user-image" >&2
	exit 1
	;;
esac

exit 0
